==> hfc-to-vue-0.0.7.tgz: package/src/index.ts <==
import {
  h,
  ref,
  watch,
  toRef,
  Teleport,
  reactive,
  onMounted,
  onUnmounted,
  defineComponent,
} from "vue";

declare class HyperFunctionComponent {
  constructor(props: HfcProps);
  static propTypes: {
    attrs?: Record<string, any>;
    events?: Record<string, Record<string, any>>;
    slots?: Record<string, Record<string, any>>;
  };

  connected(container: HTMLDivElement): void;
  changed(
    type: "attr" | "event" | "slot",
    name: string,
    oldValue: any,
    newValue: any
  ): void;
  disconnected(): void;
}

declare interface HfcProps {
  attrs: Record<string, any>;
  events: Record<string, (args: Record<string, any>) => any>;
  slots: Record<
    string,
    (container: HTMLElement | string, args: Record<string, any>) => void
  >;
}

export default function toVueComponent(
  HFC: typeof HyperFunctionComponent,
  lowercase?: boolean
) {
  const propKeys = Object.keys(HFC.propTypes.attrs || {});
  const eventKeys = Object.keys(HFC.propTypes.events || {});
  const slotKeys = Object.keys(HFC.propTypes.slots || {});

  const propKeyMap: Record<string, string> = {};
  const eventKeyMap: Record<string, string> = {};
  const slotKeyMap: Record<string, string> = {};
  if (lowercase) {
    propKeys.forEach((k) => (propKeyMap[k.toLowerCase()] = k));
    eventKeys.forEach(
      (k) =>
        (eventKeyMap["on" + k[0].toUpperCase() + k.slice(1).toLowerCase()] = k)
    );
    slotKeys.forEach((k) => (slotKeyMap[k.toLowerCase()] = k));
  } else {
    propKeys.forEach((k) => (propKeyMap[k] = k));
    eventKeys.forEach(
      (k) => (eventKeyMap["on" + k[0].toUpperCase() + k.slice(1)] = k)
    );
    slotKeys.forEach((k) => (slotKeyMap[k] = k));
  }

  const observePropKeys = Object.keys(propKeyMap);
  const observeEventKeys = Object.keys(eventKeyMap);
  const observeSlotKeys = Object.keys(slotKeyMap);

  return defineComponent({
    props: observePropKeys,
    setup(vueProps, { slots: vueSlots, attrs: vueAttrs }) {
      const container = ref<HTMLDivElement>();

      const events: Record<string, any> = {};
      observeEventKeys.forEach((key) => {
        const event = vueAttrs[key];
        if (!event) return;
        events[eventKeyMap[key]] = event;
      });

      const slots: Record<string, any> = {};

      interface RenderSlotItem {
        name: string;
        container: HTMLElement;
        slot: Function;
        args: any;
      }
      const renderedSlots: Record<string, RenderSlotItem> = reactive({});

      observeSlotKeys.forEach((key) => {
        const slot = vueSlots[key];
        if (!slot) return;
        slots[slotKeyMap[key]] = function (
          container: HTMLElement | string,
          args: any
        ) {
          if (typeof container === "string") {
            container = document.querySelector(container) as HTMLElement;
          }

          const slotItem = renderedSlots[key];
          if (!slotItem) {
            renderedSlots[key] = { name: key, container, slot, args };
            return;
          }

          if (!container) {
            delete renderedSlots[key];
            return;
          }

          slotItem.container = container;
          slotItem.args = args;
        };
      });

      const attrs: Record<string, any> = {};
      observePropKeys.forEach((key) => {
        if (vueProps[key]) attrs[propKeyMap[key]] = vueProps[key];
      });

      const hfc = new HFC({
        attrs,
        events,
        slots,
      });

      observePropKeys.forEach((key) => {
        watch(
          toRef(vueProps, key),
          (curr, prev) => {
            attrs[propKeyMap[key]] = curr;
            hfc.changed("attr", propKeyMap[key], prev, curr);
          },
          { deep: true }
        );
      });

      onMounted(() => {
        hfc.connected(container.value as HTMLDivElement);
      });

      onUnmounted(() => {
        hfc.disconnected();
      });

      return () => [
        h("div", { ref: container }),
        Object.keys(renderedSlots).map((key) => {
          const item = renderedSlots[key];
          return h(Teleport, { to: item.container }, item.slot(item.args));
        }),
      ];
    },
  });
}

==> hfc-to-vue-0.0.8.tgz: package/src/index.ts <==
import {
  h,
  ref,
  watch,
  toRef,
  Teleport,
  reactive,
  onMounted,
  onUnmounted,
  defineComponent,
} from "vue";

declare class HyperFunctionComponent {
  static propTypes?: HfcPropTypes;
  constructor(props: HfcProps);

  connected(container: HTMLDivElement): void;
  changed?(
    type: "attr" | "event" | "slot",
    name: string,
    oldValue: any,
    newValue: any
  ): void;
  disconnected?(): void;
}

declare interface HfcProps {
  attrs: Record<string, any>;
  events: Record<string, (args: Record<string, any>) => any>;
  slots: Record<
    string,
    (container: HTMLElement, args: Record<string, any>) => void
  >;
}

declare interface HfcPropTypes {
  attrs?: Record<string, any>;
  events?: Record<string, Record<string, any>>;
  slots?: Record<string, Record<string, any>>;
}

export default function toVueComponent(
  HFC: typeof HyperFunctionComponent,
  lowercase?: boolean
) {
  HFC.propTypes = HFC.propTypes || {};
  const propKeys = Object.keys(HFC.propTypes.attrs || {});
  const eventKeys = Object.keys(HFC.propTypes.events || {});
  const slotKeys = Object.keys(HFC.propTypes.slots || {});

  const propKeyMap: Record<string, string> = {};
  const eventKeyMap: Record<string, string> = {};
  const slotKeyMap: Record<string, string> = {};
  propKeys.forEach((k) => {
    propKeyMap[lowercase ? k.toLowerCase() : k] = k;
  });
  eventKeys.forEach((k) => {
    eventKeyMap[
      lowercase
        ? "on" + k[0].toUpperCase() + k.slice(1).toLowerCase()
        : "on" + k[0].toUpperCase() + k.slice(1)
    ] = k;
  });
  slotKeys.forEach((k) => {
    slotKeyMap[lowercase ? k.toLowerCase() : k] = k;
  });

  const observePropKeys = Object.keys(propKeyMap);
  const observeEventKeys = Object.keys(eventKeyMap);
  const observeSlotKeys = Object.keys(slotKeyMap);

  return defineComponent({
    props: observePropKeys,
    setup(vueProps, { slots: vueSlots, attrs: vueAttrs }) {
      const container = ref<HTMLDivElement>();

      const events: Record<string, any> = {};
      observeEventKeys.forEach((key) => {
        const event = vueAttrs[key];
        if (!event) return;
        events[eventKeyMap[key]] = event;
      });

      const slots: Record<string, any> = {};

      interface RenderSlotItem {
        name: string;
        container: HTMLElement;
        slot: Function;
        args: any;
      }
      const renderedSlots: Record<string, RenderSlotItem> = reactive({});

      observeSlotKeys.forEach((key) => {
        const slot = vueSlots[key];
        if (!slot) return;
        slots[slotKeyMap[key]] = function (
          container: HTMLElement | null,
          args: any
        ) {
          if (!container) {
            delete renderedSlots[key];
            return;
          }

          const slotItem = renderedSlots[key];
          if (!slotItem) {
            renderedSlots[key] = { name: key, container, slot, args };
            return;
          }

          if (slotItem.container !== container) {
            slotItem.container = container;
          }

          slotItem.args = args;
        };
      });

      let hfc: HyperFunctionComponent;
      const attrs: Record<string, any> = {};
      observePropKeys.forEach((key) => {
        if (!vueProps[key]) return;
        attrs[propKeyMap[key]] = vueProps[key];

        watch(
          toRef(vueProps, key),
          (curr, prev) => {
            attrs[propKeyMap[key]] = curr;
            hfc.changed?.("attr", propKeyMap[key], prev, curr);
          },
          { deep: true }
        );
      });

      hfc = new HFC({
        attrs,
        events,
        slots,
      });

      onMounted(() => {
        hfc.connected(container.value as HTMLDivElement);
      });

      onUnmounted(() => {
        hfc.disconnected?.();
      });

      return () => [
        h("div", { ref: container }),
        Object.keys(renderedSlots).map((key) => {
          const item = renderedSlots[key];
          return h(Teleport, { to: item.container }, item.slot(item.args));
        }),
      ];
    },
  });
}
